//Basic starry background

class _starfield
{
	DXWindow * scr;

	_surface * stars[19];

	LFSR_PRNG generator;

	void GetCoords(long Depth, long OriginX, long OriginY, long &DrawX, long &DrawY)
	{	// Incidentally, this is inline so there is no function call overhead
		generator.Seed(OriginX + ROT(OriginY, 13 + Depth));
		register long output = (long) (generator.GetNext() * 16384.0);

		DrawX = output % 128;
		DrawY = output >> 7;
	}

public:
	_starfield(DXWindow *);

	~_starfield();

	void Render(DWORD, DWORD);
};

_starfield::_starfield(DXWindow * screen)
{
	scr = screen;

	stars[0] = new _surface(scr, "gfx/starfield/1.bmp");
	stars[1] = new _surface(scr, "gfx/starfield/2.bmp");
	stars[2] = new _surface(scr, "gfx/starfield/3.bmp");
	stars[3] = new _surface(scr, "gfx/starfield/4.bmp");
	stars[4] = new _surface(scr, "gfx/starfield/5.bmp");
	stars[5] = new _surface(scr, "gfx/starfield/6.bmp");
	stars[6] = new _surface(scr, "gfx/starfield/7.bmp");
	stars[7] = new _surface(scr, "gfx/starfield/8.bmp");
	stars[8] = new _surface(scr, "gfx/starfield/9.bmp");
	stars[9] = new _surface(scr, "gfx/starfield/10.bmp");
	stars[10] = new _surface(scr, "gfx/starfield/11.bmp");
	stars[11] = new _surface(scr, "gfx/starfield/12.bmp");
	stars[12] = new _surface(scr, "gfx/starfield/13.bmp");
	stars[13] = new _surface(scr, "gfx/starfield/14.bmp");
	stars[14] = new _surface(scr, "gfx/starfield/15.bmp");
	stars[15] = new _surface(scr, "gfx/starfield/16.bmp");
	stars[16] = new _surface(scr, "gfx/starfield/17.bmp");
	stars[17] = new _surface(scr, "gfx/starfield/18.bmp");
	stars[18] = new _surface(scr, "gfx/starfield/19.bmp");
}

_starfield::~_starfield()
{
	for (int i = 0; i < 19; i++)
		delete stars[i];
}

void _starfield::Render(DWORD Depth, DWORD Color)
{
	if (scr->HalfWidth < 64 || scr->HalfHeight < 64)
		return;

	long Scale		= 256 >> Depth;

	long BaseX		= scr->CenterX >> Depth;
	long BaseY		= scr->CenterY >> Depth;

	long NearX		= BaseX % Scale;
	long NearY		= BaseY % Scale;

	long FarX		= (BaseX + scr->Width)  % Scale;
	long FarY		= (BaseY + scr->Height) % Scale;

	long CountX		= (scr->Width  - Scale + NearX - FarX) >> (8 - Depth);
	long CountY		= (scr->Height - Scale + NearY - FarY) >> (8 - Depth);

	long OriginX	= BaseX >> (8 - Depth);
	long OriginY	= BaseY >> (8 - Depth);

	register long x;
	register long y;

	register long DrawX;
	register long DrawY;

	scr->Lock();

	// UL
	GetCoords(Depth,
			  OriginX,
			  OriginY,
			  DrawX,
			  DrawY);

	if (NearX <= DrawX &&
		NearY <= DrawY)
			scr->SetPixel(DrawX - NearX,
						  DrawY - NearY,
						  Color, Color, Color);

	// UR
	GetCoords(Depth,
			  OriginX + CountX + 1,
			  OriginY,
			  DrawX,
			  DrawY);

	if (NearY <= DrawY)
	if (DrawX + (scr->Width - FarX) < scr->Width)
		scr->SetPixel(DrawX + (scr->Width - FarX),
					  DrawY - NearY,
					  Color, Color, Color);

	// LL
	GetCoords(Depth,
			  OriginX,
			  OriginY + CountY + 1,
			  DrawX,
			  DrawY);

	if (NearX <= DrawX)
	if (DrawY + (scr->Height - FarY) < scr->Height)
		scr->SetPixel(DrawX - NearX,
					  DrawY + (scr->Height - FarY),
					  Color, Color, Color);

	// LR
	GetCoords(Depth,
			  OriginX + CountX + 1,
			  OriginY + CountY + 1,
			  DrawX,
			  DrawY);

	if ((DrawX + (scr->Width  - FarX) < scr->Width) &&
		(DrawY + (scr->Height - FarY) < scr->Height))
			scr->SetPixel(DrawX + (scr->Width  - FarX),
						  DrawY + (scr->Height - FarY),
						  Color, Color, Color);

	for (x = 1; x <= CountX; x++)
	{
		// Top
		GetCoords(Depth,
				  OriginX + x,
				  OriginY,
				  DrawX,
				  DrawY);

		if (NearY <= DrawY)
				scr->SetPixel(DrawX - NearX + x * Scale,
							  DrawY - NearY,
							  Color, Color, Color);

		// Bottom
		GetCoords(Depth,
				  OriginX + x,
				  OriginY + CountY + 1,
				  DrawX,
				  DrawY);

		if (DrawY + (scr->Height - FarY) < scr->Height)
			scr->SetPixel(DrawX - NearX + x * Scale,
						  DrawY + (scr->Height - FarY),
						  Color, Color, Color);
	}

	for (y = 1; y <= CountY; y++)
	{
		// Left
		GetCoords(Depth,
				  OriginX,
				  OriginY + y,
				  DrawX,
				  DrawY);

		if (NearX <= DrawX)
				scr->SetPixel(DrawX - NearX,
							  DrawY - NearY + y * Scale,
							  Color, Color, Color);

		// Right
		GetCoords(Depth,
				  OriginX + CountX + 1,
				  OriginY + y,
				  DrawX,
				  DrawY);

		if (DrawX + (scr->Width - FarX) < scr->Width)
			scr->SetPixel(DrawX + (scr->Width - FarX),
						  DrawY - NearY + y * Scale,
						  Color, Color, Color);
	}

	// Fill
	for (x = 1; x <= CountX; x++)
	for (y = 1; y <= CountY; y++)
	{
		GetCoords(Depth,
				  OriginX + x,
				  OriginY + y,
				  DrawX,
				  DrawY);

		scr->SetPixel(DrawX - NearX + x * Scale,
					  DrawY - NearY + y * Scale,
					  Color, Color, Color);
	}

	scr->Unlock();
}
